SpellShader = {}

RESPONSE_TIME = 300 -- milliseconds, response time based on how fast we percieve (padded)
BUFFER_TIME = 1000 -- milliseconds, extra time added for applying less important (de)buffs 
--[[
The spell shader can apply custom effects to the Spell object. The spell object 
contains the texture and font objects, so it's easy to change them based on your
own preferences. 

There is a generic SpellShader effect which is applied to all spells which do not have 
a custom spell shader. To create a custom spell shader for a spell, simply create a 
function in the format SpellShader:_<SPELL_ID>. For example, if I wanted to apply different
heuristics to Bane of Doom, I would create this function

-- 603 is the id of Bane of Doom
function SpellShader:_603()
	self.spell.texture:SetVertexColor(0,1,0); --apply a green mask over the texture.
end

]]--
function SpellShader:new(spell)
	local object = { spell = spell, doShade = nil }
	local funcname = "_";
	if spell.id then
		funcname = funcname .. spell.id;
	end

	setmetatable (object, { __index = SpellShader } )
	
	if self[funcname] == nil then
		object.doShade = self["generic"];
	else
		object.doShade = self[funcname];
	end
	return object
end

--taste for blood
function SpellShader:_56638()
	if self.spell.timeleft > 4000 then
		self.spell:updateDuration((self.spell.timeleft - 4000) / 1000)
	else
		self.spell:updateDuration((self.spell.timeleft) / 1000)
	end 
end

--function SpellShader:_603()
--	print(	"bane of doom" ) 
--end

function SpellShader:generic()
	local cast_threshold = 0;
	if self.spell.props.spell_type == DOT then
		local down, up, lagHome, lagWorld = GetNetStats();
		cast_threshold = self.spell.props.tickTime + self.spell.props.castTime + lagWorld + RESPONSE_TIME;	
		if self.spell.timeleft <= cast_threshold - self.spell.props.tickTime - RESPONSE_TIME then
			--dot will drop off			
			self.spell.texture:SetVertexColor(1,0,0);
		elseif self.spell.timeleft <= cast_threshold then			
			--optimum time to cast gogogo
			self.spell.texture:SetDesaturated(nil);
			self.spell.texture:SetAlpha(1);
		else
			self.spell.texture:SetVertexColor(1,1,1);
			self.spell.texture:SetAlpha(SPELL_OPACITY);
			self.spell.texture:SetDesaturated(1);
		end
	elseif self.spell.props.spell_type == DEBUFF or self.spell.props.spell_type == BUFF then
		if self.spell.props.castTime then
			local down, up, lagHome, lagWorld = GetNetStats();
			cast_threshold = self.spell.props.castTime + lagWorld + RESPONSE_TIME + BUFFER_TIME;	
			if self.spell.timeleft <= cast_threshold then			
				--optimum time to cast gogogo
				self.spell.texture:SetDesaturated(nil);
				self.spell.texture:SetAlpha(1);
			else
				self.spell.texture:SetVertexColor(1,1,1);
				self.spell.texture:SetAlpha(SPELL_OPACITY);
				self.spell.texture:SetDesaturated(1);
			end
		end
	end
end

